/*
 * Hedgewars, a free turn based strategy game
 * Copyright (c) 2004-2012 Andrey Korotaev <unC0Rr@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef GAME_H
#define GAME_H

#include <QString>
#include "team.h"
#include "namegen.h"

#include "tcpBase.h"
#include "frontlib.h"

class GameUIConfig;
class GameCFGWidget;
class TeamSelWidget;

enum GameState
{
    gsNotStarted = 0,
    gsStarted  = 1,
    gsInterrupted = 2,
    gsFinished = 3,
    gsStopped = 4,
    gsDestroyed = 5,
    gsHalted = 6
};

enum RecordType
{
    rtDemo,
    rtSave,
    rtNeither,
};

class FrontLibPoller;

class HWGame : public TCPBase
{
        Q_OBJECT
    public:
        HWGame(GameUIConfig * config, GameCFGWidget * gamecfg, QString ammo, TeamSelWidget* pTeamSelWidget = 0);
        virtual ~HWGame();
        void AddTeam(const QString & team);
        void PlayDemo(const QString & demofilename, bool isSave);
        void StartLocal();
        void StartQuick();
        void StartNet();
        void StartTraining(const QString & file);
        void StartCampaign(const QString & camp, const QString & campScript, const QString & campTeam);
        void abort();
        GameState gameState;
        bool netSuspend;

    protected:
        virtual QStringList getArguments();
        virtual void onClientRead();
        void onEngineStart();

    signals:
        void SendNet(const QByteArray & msg);
        void SendChat(const QString & msg);
        void SendTeamMessage(const QString & msg);
        void GameStateChanged(GameState gameState);
        void GameStats(char type, const QString & info);
        void HaveRecord(RecordType type, const QByteArray & record);
        void ErrorMessage(const QString &);
        void campStateChanged(int);

    public slots:
        void FromNet(const QByteArray & msg);
        void FromNetChat(const QString & msg);

    private:
        enum GameType
        {
            gtLocal    = 1,
            gtQLocal   = 2,
            gtDemo     = 3,
            gtNet      = 4,
            gtTraining = 5,
            gtCampaign = 6,
            gtSave     = 7,
        };
        //char msgbuf[MAXMSGCHARS];
        QString ammostr;
        GameUIConfig * config;
        GameCFGWidget * gamecfg;
        TeamSelWidget* m_pTeamSelWidget;
        GameType gameType;

        flib_gameconn * m_conn;
        FrontLibPoller * m_poller;

        void commonConfig();
        void SendConfig();
        void SendNetConfig();
        void SendTrainingConfig();
        void SendCampaignConfig();
        void SetGameState(GameState state);
        void sendCampaignVar(const QByteArray & varToSend);
        void writeCampaignVar(const QByteArray &varVal);

        static void onChat(void* context, const char *msg, bool teamchat);
        static void onConnect(void* context);
        static void onDisconnect(void* context, int reason);
        static void onEngineMessage(void *context, const uint8_t *em, size_t size);
        static void onErrorMessage(void* context, const char *msg);
        static void onGameRecorded(void *context, const uint8_t *record, size_t size, bool isSavegame);
};

#endif
